//
//  bndblock.h
//  Ulambator v0.3
//
//  Created by Mathias Nagel on 31.05.12.
//  Copyright (c) 2012 EPFL. All rights reserved.
//

#ifndef bndblock_h
#define bndblock_h

#include "bndelement.h"

#include <string>

class BndBlock {
public:
    int nbBlocks;
    int nbPanels;
    double *allBCX, *allBCY, *allBCZ, **allXId, **allYId;
    double **allWdist, **allCdist, **allRdist;
    int *allBCtype;
    int *allSize;
    int ifid;
    int ifblock;
    int nbdrops;
    int runstep;
    double runtime;
    double *IGP, *IGW;
    int dynblock;
    int dynpanels;
    int *panel2block;
    double rlim;
    
    
    BndBlock(int blocks);
    FlowFace *Elements;
    void Init();
 
    // Intitalization of geometries
    void PanelInit(int blockId, int Size, int ofKind, int Btype, double *BCvalue, double *PosVec);
	
    // Droplet Modifications								
	void Evolve(int blockId, double XFold, double XFnew, double deltaT, double* Uvec);
	void Translate(int blockId, double, double, double);
    void correctArea(int blockId);
    void correctArea();
    void UpdateCourbure(int blockId);	// Update of the Curvature with Buffer
    void UpdateCourbure();
    void UpdateSize();
    
//	int CheckBreakUp(int blockId, double kF);
//	void PerformCut(int dropId, int cutHere1,int cutHere2);
    
	
    // Additional Subroutines
    void RemeshOn(double targetsize, int modus);
	void Stock(int blockId);

    

    void WritePos(int ObjectId, int nbSlice);
    
	void WrapInterface(int blockId);
    void EnableWrite(char* save2dir);
    void DisableWrite();
    
	typedef void (FlowFace::*FaceDist)(int dropNb);
	FaceDist DropSeed;
	void SeedDrop();
    
    
//    void WallRepuls();
//    void PerformFusion(int Drop1, int Drop2, int proxL, int proxR);
    void allStock();
    void allEvolve(double weightOldPos, double weightNewPos, double deltaT, double* Uvec);
    

    int getFixSize();
    int getAllSize();
    
 //   double DropCorrect(int Drop1, int Drop2);
 //   double DropApproach(int Drop1, int Drop2);
    
    void allWrite();
    bool writeEnabled;
    
    void DropRead(int dropId, int timestep);
    
    double getArea(int blockId);
    double getCoM(int blockId, double& Xcm, double& Ycm);
    double getHiMo(int blockId, double& Xcm, double& Ycm, double& Mcm);
    
    void LogAreaPos(int logid, int blockId);
    void LogTimeStep(int logid);
    double LogCustom(int logid, int modusid);
    

private:
    double someWay;
    int SeedOption;
    double remesh_threshold;
    std::string workdir;
    
    void DoCircle(int Size, double Radius, double* Xcir, double* Ycir, double Xpos, double Ypos, int Nmode, double Namp);// Create a circle
	void DoEgg(int, double, double, double*, double*, double, double);						// Circle similar shape
	void DoEllipse(int, double, double, double*, double*, double, double);
	void DrawLine(double*Xdraw, double *Ydraw, double X0, double Y0, double X1, double Y1, int nElm);
    void DrawSine(double*Xdraw, double *Ydraw, double X0, double Y0, double X1, double Y1, int nElm, double sinamp, int sinwaves);
	void DrawSegm(double*Xdraw, double *Ydraw, double X0, double Y0, double X1, double Y1, double R, int nElm);
    void DrawHyperbel(double*Xdraw, double *Ydraw, double X0, double Y0, double X1, double Y1, int nElm);
    
    void LoadNOrder(int blockId);
	void SeedEquiDist(int blockId, double);
//	void SeedVarDist(int blockId, double);
    
    typedef void (FlowFace::*WallObject)(int Panels, int Size, int ofKind);
    void WrapAll();
    
};


#endif
