// # Ulambator BoundaryElement class (`bndelement.cpp`) v0.3
//  This is the memory class that saves information of a boundary block

 
#include "bndelement.h"
#include <stdio.h>
#include <stdlib.h>


FlowFace::FlowFace(){
};

void FlowFace::Init(int Panels, int Size, int ofKind){
// Constructor for the boundary interfaces. Droplets and Walls
// ofKind determines the kind of boundary: 0-walls, 1-fibers, 2-drops, 3-attached interfaces

    nbPanRes = Panels;
    nbPanels  = 0;
    Xrelativ = 0.0;
    Yrelativ = 0.0;
    XE = (double*) calloc (Size+10,sizeof(double));
	YE = (double*) calloc (Size+10,sizeof(double));
    IntfX = (double*) calloc (Size+10,sizeof(double));
	IntfY = (double*) calloc (Size+10,sizeof(double));
    BCtype = (int*) calloc (Panels,sizeof(int)); 
    BCvalueX = (double*) calloc (Panels,sizeof(double));
    BCvalueY = (double*) calloc (Panels,sizeof(double));
    
	XId = (double**) calloc (Panels,sizeof(double*));
	YId = (double**) calloc (Panels,sizeof(double*));
    PanelSize = (int*) calloc (Panels,sizeof(int));
    BCkind = ofKind;
    initialArea = 0;
    
    if(BCkind>0){
        Rdist = (double*) calloc (Size+2,sizeof(double));   // distance from another droplet
        Cdist = (double*) calloc (Size+2,sizeof(double));   // distance from self infliceted crossing for break up
        Wdist = (double*) calloc (Size+2,sizeof(double));   // distance from closest wall
        KC = (double*) calloc (Size,sizeof(double));       // curvature
        for (int k=-1; k<(Size+1); k++) {
            Wdist[k] = 1e12;
            Rdist[k] = 1e12;
        }
        
    }
}

void FlowFace::Kill(){
    free(XE);
    free(YE);
    free(IntfX);
    free(IntfY);
    free(Rdist);
    free(Cdist);
    free(Wdist);
    free(KC);
    
    nbPanels = 0;
    PanelSize[0] = 0;
}

int FlowFace::getFullSize(){
    int index = 0;
    for (int k = 0; k<nbPanels; k++) {
        index += PanelSize[k];
    }
    return index; 
}

// Created by Mathias Nagel on 19.03.10.

// Copyright 2010 EPFL. All rights reserved.
