/*
 *  combase.cpp
 *  Ulambator v0.3
 *
 *  Created by Mathias Nagel on 29.01.10.
 *  Copyright 2010 EPFL. All rights reserved.
 *
 */

#include "combase.h"
#include <iostream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

void bessk01(double x, double &k0, double &k1)
{
    double t,t2,dtmp,dtmp1,i0,i1;
	
//    if (x == 0.0) {
//        k0 = 1e308;
//        k1 = 1e308;
//    }
    if (x < 3.75) {
        t = x/3.75;
        t2 = t*t;
        i0 = (((((0.0045813*t2+0.0360768)*t2+0.2659732)*t2+
				1.2067492)*t2+3.0899424)*t2+3.5156229)*t2+1.0;
        i1 = x*(((((0.00032411*t2+0.00301532)*t2+0.02658733*t2+
				   0.15084934)*t2+0.51498869)*t2+0.87890594)*t2+0.5);
    }
    else {
        t = 3.75/x;
        dtmp1 = exp(x)/sqrt(x);
        dtmp = (((((((0.00392377*t-0.01647633)*t+0.026355537)*t-0.02057706)*t+
				   0.00916281)*t-0.00157565)*t+0.00225319)*t+0.01328592)*t+0.39894228;
        i0 = dtmp*dtmp1;
        dtmp = (((((((-0.00420059*t+0.01787654)*t-0.02895312)*t+0.02282967)*t-
				   0.01031555)*t+0.00163801)*t-0.00362018)*t-0.03988024)*t+0.39894228;
        i1 = dtmp*dtmp1;
    }
    if (x < 2.0) {
        t = 0.5*x;
        t2 = t*t;    // already calculated above
        dtmp = (((((0.0000074*t2+0.0001075)*t2+0.00262698)*t2+0.0348859)*t2+
				 0.23069756)*t2+0.4227842)*t2-0.57721566;
        k0 = dtmp - i0*log(t);
        dtmp = (((((-0.00004686*t2-0.00110404)*t2-0.01919402)*t2-
				  0.18156897)*t2-0.67278578)*t2+0.15443144)*t2+1.0;
        k1 = dtmp/x + i1*log(t);
    }
    else {
        t = 2.0/x;
        dtmp1 = exp(-x)/sqrt(x);
        dtmp = (((((0.00053208*t-0.0025154)*t+0.00587872)*t-0.01062446)*t+
				 0.02189568)*t-0.07832358)*t+1.25331414;
        k0 = dtmp*dtmp1;
        dtmp = (((((-0.00068245*t+0.00325614)*t-0.00780353)*t+0.01504268)*t-
				 0.0365562)*t+0.23498619)*t+1.25331414;
        k1 = dtmp*dtmp1;
    }
}

void bessi12(double x,double &i1,double &i2)
{
    double t,t2,dtmp,dtmp1,i0;
	
    if (x == 0.0) {
        i0 = 1.0;
        i1 = 0.0;
		i2 = 0.0;
    }
    if (x < 3.75) {
        t = x/3.75;
        t2 = t*t;
        i0 = (((((0.0045813*t2+0.0360768)*t2+0.2659732)*t2+
				1.2067492)*t2+3.0899424)*t2+3.5156229)*t2+1.0;
        i1 = x*((((((0.00032411*t2+0.00301532)*t2+0.02658733)*t2+
				   0.15084934)*t2+0.51498869)*t2+0.87890594)*t2+0.5);
    }
    else {
        t = 3.75/x;
        dtmp1 = exp(x)/sqrt(x);
        dtmp = (((((((0.00392377*t-0.01647633)*t+0.026355537)*t-0.02057706)*t+
				   0.00916281)*t-0.00157565)*t+0.00225319)*t+0.01328592)*t+0.39894228;
        i0 = dtmp*dtmp1;
        dtmp = (((((((-0.00420059*t+0.01787654)*t-0.02895312)*t+0.02282967)*t-
				   0.01031555)*t+0.00163801)*t-0.00362018)*t-0.03988024)*t+0.39894228;
        i1 = dtmp*dtmp1;
    }

	i2 = i0-2.0*i1/x;
}

void dgemv(double *A, double *x, double *y, int m, int n, double alpha , double beta)
{
	int inc = 1;
	char trans = 'N';
	
	dgemv_(&trans, &m, &n, &alpha, A, &m, x, &inc, &beta, y, &inc);
}

void dgemm(double *A, double* B, double *C, int m, int n, int k, double alpha, double beta)
{
	char trans;
	
	trans = 'N';
	
	dgemm_(&trans, &trans, &m, &n, &k, &alpha, A, &m, B, &k, &beta, C, &m);
}

int dsysv(double *A, double *b, int n)
{
    int nrhs, lda, ldb, info;
	int *ipiv;
	int status = 0;
    char uplo = 'U';
	double *work;
    int lwork = int(n*n/10);  // TEST ceil()
    work = new double[lwork];
	nrhs = 1; /* The Fortran routine that I am calling will solve a
			   system of equations with multiple right hand sides.
			   nrhs gives the number of right hand sides, and then b
			   must be a matrix of dimension n by nrhs. */
	
	lda = n; // The leading dimension of A
	
	ipiv = new int[n]; // Allocate memory to store pivot information
	
	ldb = n;
	
	/* The Fortran routine replaces the input information in b with the
     results.  Since we are interested in the results, we put the
     initial conditions in the output array and use that in the
     call. */
	
    //	for (int i=0; i<n; i++)
	
    //	x[i] = b[i];
	
	// Now the function call
	info = 0;
    
    dsysv_(&uplo, &n, &nrhs, A, &lda, ipiv, b, &ldb, work, &lwork, &info);
	
	// Clean up the memory before returning to the calling program
    assert(info==0);
    assert(b[0]==b[0]);
	delete ipiv;
	return status;
}



int dgesv(double *A, double *b, int n)
{
	int nrhs, lda, ldb, info;
	int *ipiv;
	int status = 0;
	
	nrhs = 1; /* The Fortran routine that I am calling will solve a
			   system of equations with multiple right hand sides.
			   nrhs gives the number of right hand sides, and then b
			   must be a matrix of dimension n by nrhs. */
	
	lda = n; // The leading dimension of A
	
	ipiv = new int[n]; // Allocate memory to store pivot information
	
	ldb = n;
	
	/* The Fortran routine replaces the input information in b with the
     results.  Since we are interested in the results, we put the
     initial conditions in the output array and use that in the
     call. */
	
//	for (int i=0; i<n; i++) 
	
//	x[i] = b[i];
	
	// Now the function call
	info = 0;
	dgesv_(&n, &nrhs, A, &lda, ipiv, b, &ldb, &info);
	
	// Clean up the memory before returning to the calling program
    assert(info==0);
    assert(b[0]==b[0]);
	delete ipiv;
	return status;
}



int dgsev2rhs(double *A, double *b1, double *b2, int n)
{
	int nrhs, lda, ldb, info;
    double *b;
	int *ipiv;
	int status = 0;
	
	nrhs = 2; /* The Fortran routine that I am calling will solve a
			   system of equations with multiple right hand sides.
			   nrhs gives the number of right hand sides, and then b
			   must be a matrix of dimension n by nrhs. */
	
	lda = n; // The leading dimension of A
	
	ipiv = new int[n]; // Allocate memory to store pivot information
	b = new double[2*n];
    
    for (int j=0; j<n; j++) {
        b[j] = b1[j];
        b[j+n] = b2[j];
    }
    
	ldb = n;
	
	/* The Fortran routine replaces the input information in b with the
     results.  Since we are interested in the results, we put the
     initial conditions in the output array and use that in the
     call. */
	
    //	for (int i=0; i<n; i++) 
	
    //	x[i] = b[i];
	
	// Now the function call
	info = 0;
	dgesv_(&n, &nrhs, A, &lda, ipiv, b, &ldb, &info);
	
	// Clean up the memory before returning to the calling program
	assert(info==0);
    assert(b[0]==b[0]);
    for (int j=0; j<n; j++) {
        b1[j] = b[j];
        b2[j] = b[j+n];
    }
    
	delete ipiv;
	return status;
}

int dgels(double *A, double *b, int m, int n, double *x)
{
	int nrhs, lda, ldb, info;

	int lwork;
	int status =0;
	
	char trans = 'N';
	nrhs = 1; /* The Fortran routine that I am calling will solve a
			   system of equations with multiple right hand sides.
			   nrhs gives the number of right hand sides, and then b
			   must be a matrix of dimension n by nrhs. */
	
	lda = m; // The leading dimension of A

	lwork = 2*n*m;
	
	ldb = m;
	
	/* The Fortran routine replaces the input information in b with the
     results.  Since we are interested in the results, we put the
     initial conditions in the output array and use that in the
     call. */
	
	for (int i=0; i<m; i++) x[i] = b[i];
	
	// Now the function call
	
	dgels_(&trans, &m, &n, &nrhs, A, &lda, x, &ldb, b, &lwork, &info);
	
	// Clean up the memory before returning to the calling program
	assert(info==0);

	return status;
}

int dgeinv(double *A, int N){
    int info;
    int *ipiv;
    ipiv = new int[N]; // Allocate memory to store pivot
    double *work;
    int lwork = int(N*N*1.1);  // TEST ceil()
    work = new double[lwork];
    
    dgetrf_(&N, &N, A, &N, ipiv, &info);
    dgetri_(&N, A, &N, ipiv, work, &lwork, &info);
    
    delete work;
    return info;

}

int schur(double *A, double *B, double *C, double *D, double *f1, double *f2, int M, int N){
    double *work, *vork;
    int info;
    work = new double[M*N];
    vork = new double[M];
    
    // Matrix multiplications
    dgemm(A, B, work, M, N, M, 1.0, 0.0);
    dgemm(C, work, D, N, N, M, -1.0, 1.0);
    
    // Vector multiplications
    dgemv(A, f1, vork, M, M, 1.0, 0.0);
    dgemv(C, vork, f2, N, M, -1.0, 1.0);
    
    info = dgesv(D, f2, N);
    
    delete work;
    delete vork;
    return info;
}

int dgesch(double *A, double *f1, int M, int N){
    double *work, *vork;
    int info;
    double *B, *C, *D, *f2;
    work = new double[M*N];
    vork = new double[M];
    C = &A[M*M];
    B = &A[M*(N+M)];
    D = &A[M*(M+2*N)];
    f2 = &f1[M];
    
    // Matrix multiplications
    dgemm(A, B, work, M, N, M, 1.0, 0.0);
    dgemm(C, work, D, N, N, M, -1.0, 1.0);
    
    // Vector multiplications
    dgemv(A, f1, vork, M, M, 1.0, 0.0);
    dgemv(C, vork, f2, N, M, -1.0, 1.0);
    
    info = dgesv(D, f2, N);
    
    delete work;
    delete vork;
    return info;
}


void dg_ctof(double **in, double *lout, int rows, int cols)
{
	int i, j;
	
	for (i=0; i<rows; i++) {
		for (j=0; j<cols; j++) {
			lout[i+j*rows] = in[i][j];
		}
	}
}

