/*
 *  matrixblock.h
 *  Ulambator v0.7
 *
 *  Created by Mathias Nagel on 23.03.10.
 *  Copyright 2010 EPFL. All rights reserved.
 *
 */

#ifndef MATRIXBLOCK_H
#define MATRIXBLOCK_H

#include "bndblock.h"
#include <string>

class MatrixBlock {
public:
	MatrixBlock(BndBlock *bembl, double aspectRatio, double viscRatio, double capillaryNumber);
	

//	virtual void vprobe(int side, double Xpos, double Ypos, double& U, double& V, double &P){};
    // Solvers	
	int RunOne();
	int SolveRK1();
	int SolveRK2();
//    int SolveFilm();
//    int RunRigidRK1(double dgap, double* move);
//    int RunRigidRK2(double dgap, double *move);
//    int RunRigidManual(double dgap, double *move);
	void Asifstep(int blockId, double& Xcm, double& Ycm);
	void Solve();
    
	// Output Routines
    void VisMatlab(int nbSlice, int resolution, double left, double right, double bottom, double top);
    void VTKexport(int nbSlice, int resolution, double left, double right, double bottom, double top);
	void BoosThessPrintAnalytic();
	void Result(int fid);// Print result on screen
	void saveMatrix();
    void writeRHS();
	void PV(int);
	void WriteTracers(int);
	
    void SensorEnable(int sid);
    void Build(double deltatime);
//	int rsp;
	int dropNb;
    
    void EnableWrite(char* save2dir);
    void DisableWrite();
    void LogPrepare(int logid, char* disclaimer);
    void LogRuntime(int logid);
    void LogCfg();
    void LogCustom(int logid, int modusid);

    void ChoseDynBC(int choice, double para1, double para2);
    void ChoseStatBC(int choice, double para1, double para2);
    void setTimeStep(double dT, int nSteps);
    void SensorWrite(int sid, double Xp, double Yp);
    double *rhs;
    int nOFm;
    double timefactor;
    int zerotime;
    // Size of Matrix
//    virtual void ForceFree(int fblock, double dEta, double dgap){};
    void SubFolder(char* subdir);
    void BuildVisual(int points, double* Xpos, double *Ypos, double *U, double *V, double *P, int *Dom);

    int UpdateNOFM();
    void PreCondense();
    void DistributeTracers(int thenumTr, double Xd, double Yd, double Dd);
    void AdvanceTimestep(double deltaT, int modus);


protected:
    BndBlock *bem;
// Matrix Operations
											// Build matrix with following BCs
	virtual void CircTrac(int,double){};
//    virtual void BordTrac(int,double){};    // Boundary condition for droplet (type 666)
    virtual void BoosThess(int){};							// Boundary (no type)
	virtual void ZeroVel(int){};							// Boundary condition for walls  (type 0)
	virtual void NormVel(int, double, double,double){};				// Boundary condition for n/t flow (type 1)
	virtual void FlatVel(int, double, double,double, double){};
    virtual void POX(int, double, double) {};
	virtual void Pressure(int, double,double){};						// Boundary condition (type 2)
	virtual void Poiseuille(int, double,double){};					// Boundary condition (type 3)
    virtual void SineFlow(int, double, double,double){};
    
    virtual void SZeroVel(int){};							// Boundary condition for walls  (type 0)
	virtual void SNormVel(int, double, double,double){};				// Boundary condition for n/t flow (type 1)
	virtual void SFlatVel(int, double, double,double, double){};
    virtual void SPOX(int, double, double) {};
	virtual void SPressure(int, double,double){};						// Boundary condition (type 2)
	virtual void SPoiseuille(int, double,double){};
    virtual void SCircTrac(int,double){};
//    virtual void SBordTrac(int,double){};
    virtual void SSineFlow(int, double, double,double){};
    virtual void getForce(int, int, double*, double*){};
   
    virtual void InfiniBox(double xvel, double yvel){};
    virtual void InfiniSource(double qdebit, double xpos, double ypos, double dEta){};
    virtual void InfiniLinearFlow(double lin, double colin, double dEta){};
    virtual void FiniLinearFlow(int, double lin, double colin, double dEta){};
    virtual void VisuZeroVel(int inP,double *Xpos, double *Ypos,int points, double *U, double *V, double *P){};
    virtual void VisuNormVel(int inP,double *Xpos, double *Ypos, double varA, double varB, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuFlatVel(int inP,double *Xpos, double *Ypos, double varA, double varB, double Theta, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuPressure(int inP,double *Xpos, double *Ypos, double varA, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuPoiseuille(int inP,double *Xpos, double *Ypos, double varA, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuCircTrac(int inP,double *Xpos, double*Ypos, int points, double *U, double *V, double *P){};
//    virtual void VisuBordTrac(int inP,double *Xpos, double*Ypos, int points, double *U, double *V, double *P){};
    virtual void VisuInfiniBox(double *Xpos, double *Ypos, double uvel, double vvel, int points, double *U, double *V, double *P, int *Dom){};
    virtual void VisuInfiniSource(double *Xpos, double *Ypos, double qdebit, double xpos, double ypos, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuLinearFlow(double *Xpos, double *Ypos, double lin, double colin, int points, double *U, double *V, double *P, int *Dom){};
    virtual void VisuFiniFlow(int inP,double *Xpos, double *Ypos, double varA, double varB, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuPOX(int inP,double *Xpos, double *Ypos, double varA, double dEta, int points, double *U, double *V, double *P){};
    virtual void VisuSineFlow(int inP,double *Xpos, double *Ypos, double varA, double varB, double dEta, int points, double *U, double *V, double *P){};
//    virtual void FilmTrac(double *erhs){};
    
    void MatMult();
     
    
// Additional Subroutines

    
	void computeGT(int dolog, double x1, double y1, double& G11, double& G12, double& G22, double& T111, double& T112, double& T122, double& T211, double& T212, double& T222);
	void computeGO(int dolog, double x1, double y1, double& G11, double& G12, double& G22);
    void computeGTH(int dolog, double x1, double y1, double& G11, double& G12, double& G22, double& T111, double& T112, double& T122, double& T211, double& T212, double& T222);
    void computeGOH(double x1, double y1, double& G11, double& G12, double& G22);
	void computeMP( double x1, double y1, double &M1, double &M2, double &P11, double &P12, double &P22);

    double LocalGam(int dropId, int nL, int nR);
    
    void Flatborder(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
    void Standard(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);    
    void Hole(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
    void HoleArray(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
	void Rail(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
	void Meiburg(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
	void Wedge(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
	void Marche(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
	void Rotate(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
    void Gravity(double Xp, double Yp, double Nx, double Ny, double Unorm, double Kc, double& f1, double& f2);
    void CustomStatBC(double Xp, double Yp, double Nx, double Ny, double Unorm, double gamma, double& f1, double& f2);
    double SpecialForces(int DropId, int Vertex);
    
	double *A;							// Matrix
								// right hand side
	double* u1vec;
    double* u2vec;// solution vector
    double *IGP;
	double *IGW;
    double *GP3;
    double *GW3;
	double *xTracers, *yTracers;
	double Lambda;							// viscosity
	double kF;							// k Faktor for Brinkmann

    double deltaT0;
	double channelheight;
	double DropUvel;
	double DropVvel;
    double dominantForce;       // Keeps the largest force found during one iteration
	
    int dynblock;
    int dynpanel;
	int nCK;
	int nbTracers;
	int nSubSteps;
    int prcndsing;
	
 	int ifblock;
    int nbBlocks;
    int fixsize;
    int thedropbc;
    int thestatbc;
    bool writeEnabled;
   
    double parabc1, parabc2, statbc1, statbc2;
    double widerange;
    double midrange;
    
	int *bndtype;
    double *bndvalue1;
	double *bndvalue2;
	std::string workdir;
	typedef void (MatrixBlock::*BCObject)(double Xp, double Yp, double Nx , double Ny, double Unorm, double Kc, double& f1, double& f2);
	BCObject applybc;
    int arrest;
    double Ca;                                          // Surface tension
    
private:
    FILE* logfile;
    void IntForce(int dropId);
	void ResetMR(double*, double*);
	
	double PutToRest(int dropId);
    double DetTime();
	
    double Residuum(double *sol, double &omega, double oldcvg);
    double Velocities(int dropId);
    
	double correctArea();
	int doArCo;
	int memrsv;
	double senseX;
	double senseY;
    
    int FigInOut(double xp, double yp, double crit);
	int prec;							// Refinement parameter
	int ovhd;
	

	
};


#endif
