# A module to be used when plotting Ulambator output with Python.
# An accompanying ulam.PythonPlot.py file contains commetns of how to use it
# Original version Sam Dehaeck at Universite Libre de Bruxelles
# Created 19th November 2015
import os
import re
import glob
import numpy as np
import pylab

### Typical regex would be "drop0ts(\d+).dat", placing the brackets will make the sort based on captured integer
def findFiles(regex,dirName=os.curdir):
	filenames=glob.glob(os.path.join(os.path.abspath(dirName),"*"))
	if len(filenames) == 0:
		print "No pictures found"
		return (0,None)
	# two steps, first get a list of those filenames matching the given regex
	newList=[]
	numList=[]
	for gfile in filenames:
		mat=re.search(regex,gfile)
		if (mat):
			newList.append(gfile)
			if (len(mat.groups())>0):
				nr=int(mat.groups()[-1])
				numList.append(nr)
			
	# now sort the files according to their number. Always fitting regex \d+.dat to extract number for each file
	if (len(numList)>0):
		sI=np.argsort(numList)
		sortF= [newList[i] for i in sI]
	else:
		newList.sort()
		sortF=newList
	return sortF

def readBoundary(filename):
	I=np.loadtxt(filename)
	return I

def plotBoundary(B,fmt='o',lab="ExtBoundary"):
	pylab.plot(B[1:,0],B[1:,1],fmt,label=lab)
	pylab.axis('equal')
	
def readAllBoundaries(regex,dirName=os.curdir):
	fList=findFiles(regex,dirName)
	Blist=[]
	for f in fList:
		I=np.loadtxt(f)
		Blist.append(I)
	BB=np.asarray(Blist)
	return BB

def plotAllBoundaries(BB,fmt='o',pauseSec=0.01,start=0,stop=-1,step=1):
	if (stop==-1):
		stop=len(BB)
	gI=np.arange(start,stop,step)
	for i in gI:
		b=BB[i]
		pylab.plot(b[1:,0],b[1:,1],fmt,label="Boundary-{}".format(i))
		pylab.pause(pauseSec)
	pylab.axis('equal')
	
# plotting the result here is just for instance imshow(P,clim=(0,10000))
def readViz(filename):
    F=open(filename,mode="r")
    F.readline()
    H=F.readline()
    T=(H.strip().split())
    r=int(T[0])
    c=int(T[1])
    bottom=float(T[2])
    left=float(T[3])
    top=float(T[4])
    right=float(T[5])
    extent=(left,right,bottom,top)
    
    I=np.loadtxt(filename,skiprows=2)
    K=I.reshape((r,c,-1))
    U=K[:,:,0].T
    V=K[:,:,1].T
    P=K[:,:,2].T
    X=K[:,:,3].T
    Y=K[:,:,4].T
    return X,Y,U,V,P,extent

 
